function [ all_data, g, ts ] = projReachToyDAE(accuracy)
% projReachToyDAE: Compute reachability for a 2D toy DAE system on manifold.
%
%   [ all_data, g, ts ] = projReachToyDAE(accuracy)
%  
% This function computes reachability on the constraint manifold for a toy
% DAE system in two dimensions:
%
%        x_1 = \sin \pi x_2           (algebraic constraint)
%   \dot x_2 = (1.5 + x_1)            (differential equation)
%
% We define a coordinate system on the manifold as z = x_2, and then solve a
% 1D HJ PDE.  Because we are only working in 1 spatial dimension, we save
% the entire level set function in both space and time, and then visualize
% the reach set in t vs z space (along with a numerical solution of the DAE
% for comparison).
%
% This function was originally designed as a script file, so most of the
% options can only be modified in the file.  For example, edit the file to
% change the boundary conditions, flow field parameters, etc.
%
% Input parameters:
%
%   accuracy: Controls the order of approximations.
%
%                  'low'         Use odeCFL1 and upwindFirstFirst (default).
%                  'medium'      Use odeCFL2 and upwindFirstENO2.
%                  'high'        Use odeCFL3 and upwindFirstENO3.
%                  'veryHigh'    Use odeCFL3 and upwindFirstWENO5.
%
% Output Parameters:
%
%   all_data: Solution.  The first dimension is  is time (indexed by ts) and
%   the second is space (indexed by g.xs).
  
%   g: Grid structure on which data was computed (a 1D grid).
%
%   ts: Vector of times at which the solution was computed.

% Copyright 2007 Ian M. Mitchell (mitchell@cs.ubc.ca).
% This software is used, copied and distributed under the licensing 
%   agreement contained in the file LICENSE in the top directory of 
%   the distribution.
%
% Ian Mitchell, 8/10/2007

%---------------------------------------------------------------------------
% You will see many executable lines that are commented out.
%   These are included to show some of the options available; modify
%   the commenting to modify the behavior.
  
%---------------------------------------------------------------------------
% Make sure we can see the kernel m-files.
run('../addPathToKernel');

%---------------------------------------------------------------------------
% Integration parameters.
tMax = 3.0;                  % End time.
plotSteps = 61;              % How many intermediate plots to produce?
t0 = 0;                      % Start time.
singleStep = 0;              % Plot at each timestep (overrides tPlot).

% Period at which intermediate plots should be produced.
tPlot = (tMax - t0) / (plotSteps - 1);

% How close (relative) do we need to get to tMax to be considered finished?
small = 100 * eps;

% Plot in separate subplots (set deleteLastPlot = 0 in this case)?
useSubplots = 0;

%---------------------------------------------------------------------------
% What level set should we view?
level = 0;

% How many steps of reinitialization after each timestep?
reinitSteps = 0;

% Pause after each plot?
pauseAfterPlot = 0;

% Delete previous plot before showing next?
deleteLastPlot = 0;

% What kind of display?
displayType = 'plot';

%---------------------------------------------------------------------------
% Create the grid.
g.dim = 1;
g.dx = 1 / 50;
g.min = [ -0.3 ];
g.max = [ +3.7 ];
g.bdry = @addGhostExtrapolate;
g = processGrid(g);

%---------------------------------------------------------------------------
if(nargin < 1)
  accuracy = 'medium';
end

% Choose approximations at appropriate level of accuracy.
switch(accuracy)
 case 'low'
  derivFunc = @upwindFirstFirst;
  integratorFunc = @odeCFL1;
 case 'medium'
  derivFunc = @upwindFirstENO2;
  integratorFunc = @odeCFL2;
 case 'high'
  derivFunc = @upwindFirstENO3;
  integratorFunc = @odeCFL3;
 case 'veryHigh'
  derivFunc = @upwindFirstWENO5;
  integratorFunc = @odeCFL3;
 otherwise
  error('Unknown accuracy level %s', accuracy);
end

%---------------------------------------------------------------------------
% Choose the flow field.

% Velocity field.
v = cell(g.dim, 1);
v{1} = 1.5 + sin(pi * g.xs{1});

% Set up the convection routine.
schemeFunc = @termConvection;

schemeData.velocity = v;
schemeData.derivFunc = derivFunc;
schemeData.grid = g;

%---------------------------------------------------------------------------
% Set up time approximation scheme.
integratorOptions = odeCFLset('factorCFL', 0.9, 'stats', 'on');

if(singleStep)
  integratorOptions = odeCFLset(integratorOptions, 'singleStep', 'on');
end

%---------------------------------------------------------------------------
% Create initial conditions: Plane at x = 0.
data0 = g.xs{1};
data = data0;

% An array to store the results for all time.
all_data = zeros(plotSteps, g.N);
ts = zeros(plotSteps, 1);

%---------------------------------------------------------------------------
% Restrict the Hamiltonian so that reachable set only grows.  
%   The regular approximation scheme MUST already be completely set up.

if(0)
  % It turns out that for this set of dynamics, we don't need this code.
  innerFunc = schemeFunc;
  innerData = schemeData;
  clear schemeFunc schemeData;

  % Wrap the dynamics inside the term approximation restriction routine.
  schemeFunc = @termRestrictUpdate;
  schemeData.innerFunc = innerFunc;
  schemeData.innerData = innerData;
  schemeData.positive = 0;
end

%---------------------------------------------------------------------------
% Set up reinitialization if necessary.
if(reinitSteps)
  % Register reinitialization postTimestep routine.
  integratorOptions = odeCFLset(integratorOptions, ...
                                'PostTimestep', @postTimestepReinit);
  
  % Add necessary components to schemeData structures.
  schemeData.reinitAccuracy = accuracy;
  schemeData.reinitSteps = reinitSteps;
end

%---------------------------------------------------------------------------
% Initialize Display
f = figure;
plotNum = 1;

% Set up subplot parameters if necessary.
if(useSubplots)
  rows = ceil(sqrt(plotSteps));
  cols = ceil(plotSteps / rows);
  subplot(rows, cols, plotNum);
end

% Show the evolving level set.
h = visualizeLevelSet(g, data, displayType, level, [ 't = ' num2str(t0) ]);
hold on;

%---------------------------------------------------------------------------
% Loop until tMax (subject to a little roundoff).
tNow = t0;
startTime = cputime;
ts(plotNum) = tNow;
all_data(plotNum, :) = data;
while(tMax - tNow > small * tMax)

  % Reshape data array into column vector for ode solver call.
  y0 = data(:);

  % How far to step?
  tSpan = [ tNow, min(tMax, tNow + tPlot) ];
  
  % Take a timestep.
  [ t, y, schemeData ] = feval(integratorFunc, schemeFunc, tSpan, y0,...
                               integratorOptions, schemeData);
  tNow = t(end);
  plotNum = plotNum + 1;
  ts(plotNum) = tNow;

  % Get back the correctly shaped data array
  data = reshape(y, g.shape);
  all_data(plotNum, :) = data;
  
  if(pauseAfterPlot)
    % Wait for last plot to be digested.
    pause;
  end

  % Delete last visualization if necessary.
  if(deleteLastPlot)
    delete(h);
  end

  % Move to next subplot if necessary.
  if(useSubplots)
    subplot(rows, cols, plotNum);
    hold on;
  end

  % Create new visualization.
  h = visualizeLevelSet(g, data, displayType, level, [ 't = ' num2str(tNow)]);
  drawnow;
 
end

endTime = cputime;
fprintf('Total execution time %g seconds\n', endTime - startTime);

%---------------------------------------------------------------------------
% Visualize the results in a single plot.
figure;

% HJ PDE solution approximation.
[ t_grid, x_grid ] = ndgrid(ts, g.vs{1});
[ garbage, hc ] = contour(t_grid, x_grid, all_data, [ 0 0 ], 'b-');
hold on;

% DAE solution approximation.
[ t_dae, y_dae ] = simulateToyDAE(0);
hp = plot(t_dae, y_dae(:,2), 'r--');

% Make a pretty plot.
axis([ 0, tMax, g.min, g.max ]);
daspect([ 1 1 1 ]);
setProperty([ hc; hp ], { 'LineWidth' }, { 2 });
% We adjust font size on the axes mostly so that the legend will be visible.
set(gca, 'FontSize', 15);
xlabel('t');  ylabel('x');
[ garbage, hl ] = legend('HJ PDE', 'DAE', 'Location', 'NorthWest');



%---------------------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%---------------------------------------------------------------------------
function setProperty(handle, propertyNames, propertyValues)
% setProperty: set handle graphics property settings.
%
%   setProperty(handle, propertyNames, propertyValues)
%
% For a given handle graphics object, sets one or more of that object's
%   graphics properties to particular values.
%
% Parameters:
%   handle          Graphics object handle(s).
%   propertyNames   Cell vector of names of the properties to be set.
%   propretyValues  Cell vector of values of the corresponding properties.

for i = 1 : length(propertyNames)
  for j = 1 : length(handle)
    set(handle(j), propertyNames{i}, propertyValues{i});
  end
end
drawnow;

