function [ ts ys ] = simulateOneGen(num_traj, simulate_DAE, ...
                                    plot_proj, plot_constraint)
% simulateOneGen: simulates the simple power system model.
%
%   [ ts ys ] = simulateOneGen(num_traj, simulate_DAE, ...
%                              plot_proj, plot_constraint)
%
% This function simulates a simple 3D DAE model of the power system from:
%
%   Venkatasubramanian, Schattler & Jaborszky, "Voltage Dynamics: Study
%   of a Generator with Voltage Control, Transmission, and Matched MW
%   Load," IEEE Trans. Automatic Control, v. 37, n. 11, pp. 1717-1733
%   (November, 1992).
%
% with parameters from Susuki & Hikihara's HSCC 2006 poster, "Application
% of Hybrid System Theory to Power System Voltage Stability Analysis," or
% from figures 4 & 8 of Venkatasubramanian.
%  
% Simulation is performed using Matlab's ode15s routine and a singular mass
% matrix.  Occasionally the error control component of this routine will
% produce a warning: "Failure at t=????.  Unable to meet integration
% tolerances without reducing the step size below the smallest value allowed
% ..."  This failure occurs if the integrator tries to take a step across
% the singularity.  I have tried to reduce the chance of this happening by
% introducing the event detection and reducing the maximum step size, but it
% still occasionally crops up.
%
% Input Parameters:
%
%   num_traj - Integer.  Number of trajectories to plot.  Default = 16.
%
%   simulate_DAE - Boolean.  Should we use the true DAE system? Otherwise
%   use the ODE approximation.  The latter attempts to keep the constraint
%   g(E,Ep) = 0 by the approximation dg/dt = 0.  Default = true.
%
%   plot_proj - Boolean.  Create a separate plot that shows the projection
%   of the trajectories onto the E - E_fd space.  Default = false.
%
%   plot_constraint - Boolean.  Create a separate plot that shows how
%   well the constraint is satisfied?  Default = false.
%
% Output Parameters:
%
%   ts: Cell vector of column vectors.  Each element of the cell vector
%   is the time sample return vector from one of Matlab's integrators.
%
%   ys: Cell vector of arrays.  Each element of the cell vector is the state
%   sample return array from one of Matlab's integrators; each row of the
%   array is the state at the corresponding row of the corresponding time
%   vector.

% Copyright 2006 Ian M. Mitchell (mitchell@cs.ubc.ca).
% This software is used, copied and distributed under the licensing 
%   agreement contained in the file LICENSE in the top directory of 
%   the distribution.
%
% Ian Mitchell, 4/23/06

  % How many transmission routes?  Parameters are designed for 1 or 2.
  % Venkatasubramanian uses 2 throughout.
  transmission_routes = 2;

  % Use which parameter set?  Note that the axis bounds and maximum time
  % may need to be adjusted.
  useSusuki = 0;

  % Style used for the trajectory lines.
  plot_style = 'b-';
  
  %---------------------------------------------------------------------------
  % How many sample trajectories?
  if(nargin < 1)
    num_traj = 16;
  end
  
  % Do we want the DAE or the ODE model?
  if(nargin < 2)
    simulate_DAE = 1;
  end

  % Do we want to plot a projection into the E - E_fd space?
  if(nargin < 3)
    plot_proj = 0;
  end

  % Do we want to plot how well the constraint is satisfied?
  if(nargin < 4)
    plot_constraint = 0;
  end

  if(nargout > 0)
    ts = cell(num_traj, 1);
  end
  
  if(nargout > 1)
    ys = cell(num_traj, 1);
  end
  
  %---------------------------------------------------------------------------
  % Problem parameters common to both Susuki and Venkatasubramanian.
  p.Tp_d0 = 5;
  p.E_r   = 1.0;
  p.H     = 0;
  p.x_d   = 1.2;
  p.B     = 0;
  p.xp_d  = 0.2;
  p.K     = 7.0;

  % Problem parameters that differ.
  if(useSusuki)
    p.T     = 1;
    p.E0_fd = 2.0;
    p.P     = 0.9;
  else
    p.T     = 1.5;
    p.E0_fd = 1.6;
    % Choice of P determines which subfigure of figure 8 is in effect.
    p.P     = 1.0;
  end
  
  % Calculated parameters.
  p.x     = 0.2 / transmission_routes;
  p.xp    = p.x + p.xp_d;
  
  p.Q0    = 0.5 * p.P;
  if((p.H == 0) && (p.B == 0))
    p.Q   = p.Q0;
  else
    error('DAE model is not currently set up to handle nonconstant Q');

    % The problem is that Q will now depend on state variable E, so the code in
    % the dynamics routine would have to be modified.  Not a huge deal for
    % the DAE model, but the fully ODE model would have to be rederived.
    %
    % Q = p.Q0 + p.H * E + p.B * E.^2
  end

  % Reachability parameters.  Note that changing this value may not change the
  % actual initial conditions -- it all depends on what the DAE simulator
  % finds as consistent IC.
  p.E_min = 0.70;
  
  %---------------------------------------------------------------------------
  % Time interval.  Equal spacing to see that the system slows down on the
  % left hand side of a phase space plot.
  tspan = [ 0, 5 ];

  % Coordinate ordering chosen to align with plots in cited papers.  This
  % results in the odd situation where the algebraically constrained
  % variable is first.

  %---------------------------------------------------------------------------
  if(simulate_DAE)
    % Mass matrix.
    M = [ 0, 0, 0; 0, 1, 0; 0, 0, 1 ];
    M_singular = 'yes';
    % Dynamics
    dynamics_func = @dynamicsDAE;
  else
    M = eye(3);
    M_singular = 'no';
    % Dynamics
    dynamics_func = @dynamicsODE;
  end
    
  % At some point we should try to calculate the Jacobian.
  
  % Set the options that are the same for all simulations.
  options = odeset('Mass', M, 'MassSingular', M_singular, 'Vectorized', 'on');

  % If you want to watch the trajectories during simulation.  The
  % downside is that it mucks up the buttons on the figure window.
  %
  %options = odeset(options, 'OutputFcn', 'odephas3');

  % Detects that the trajectory is getting close to the singular surface
  % and ends the simulation.
  options = odeset(options, 'Events', @detectSingular);

  % We were also getting very big stepsizes, so I capped them
  options = odeset(options, 'MaxStep', 0.1);
  
  %---------------------------------------------------------------------------
  % Initial conditions, representing the boundary of the unsafe set.
  % To get actual initial conditions, we'd have to solve the algebraic
  % constraint.  For the DAE version, we just need to get close and the
  % integrator will do the rest.  For the ODE version, errors in the
  % initial conditions will never be corrected.
  E_init = p.E_min * ones(num_traj, 1);

  switch(transmission_routes)
   case 1
    Ep_init = 1.0866 * ones(num_traj, 1);
    %Ep_init = 1.0 * ones(num_traj, 1);
   case 2
    Ep_init = 1.00975 * ones(num_traj, 1);
   otherwise
    error('Unknown initial conditions for %d transmission routes', ...
          transmission_routes);
  end
    
  E_fd_init = linspace(1, 4, num_traj);

  %---------------------------------------------------------------------------
  % Set up the trajectories.
  f1 = figure;
  hold on;
  if plot_proj
    f3 = figure;
    hold on;
  end
  if plot_constraint
    f2 = figure;
    hold on;
  end

  % Show the relevant surfaces.
  figure(f1);
  [ p.E_sing, p.Ep_sing ] = visualizeSurfaces(p);
  
  for i = 1 : num_traj

    % Regular initial condition.
    y0 = [ E_init(i); Ep_init(i); E_fd_init(i) ];

    % To track the behavior of the constraint if we are using the ODE
    % formulation.
    if(~simulate_DAE)
      % Extract the constraint from the DAE dynamics code.
      yp0 = dynamicsDAE(0, y0, p);
      p.g0 = yp0(1);
    end
    
    % At some point we should try to calculate the initial slope (to help the
    % DAE solver).
    %
    % yp0 = 
    % options = odeset(options, 'InitialSlope', yp0);

    % Give it a whirl.
    [ t, y ] = ode15s(dynamics_func, tspan, y0, options, p);

    if(nargout > 0)
      ts{i} = t;
    end
    
    if(nargout > 1)
      ys{i} = y;
    end
        
    % Phase Space.
    figure(f1);
    h1 = plot3(y(:,1), y(:,2), y(:,3), plot_style);
    
    % E vs E_fd projection.
    if plot_proj
      figure(f3);
      h2 = plot(y(:,1), y(:,3), plot_style);
    else
      h2 = [];
    end
    
    % Show level of constraint satisfaction.
    if plot_constraint
      figure(f2)
      yp = dynamicsDAE(t, y', p);
      h3 = plot(t, yp(1,:), plot_style);
    else
      h3 = [];
    end

    % Make the lines thick enough to see.
    set([ h1; h2; h3 ], 'LineWidth', 2);
    
  end

  %---------------------------------------------------------------------------
  % Pretty up the results.
  figure(f1);
  xlabel('E');  ylabel('E`');  zlabel('E_{fd}');
  view(3);

  axis tight;

  if plot_proj
    figure(f3);
    xlabel('E');
    ylabel('E_{fd}');
  end
  
  if plot_constraint
    figure(f2)
    xlabel('t');
    ylabel('g(E,Ep)');
  end

  

%---------------------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%---------------------------------------------------------------------------
function ydot = dynamicsDAE(t, y, p)
% dynamicsDAE: implements the DAE for the single generator and load system.
%
% p is a structure containing the system parameters.
%
% This model assumes that Q = Q_0 + H E + B E^2 = Q_0 is constant
% (ie H = B = 0).  Modification would involve replacing p.Q below with
% a state dependent vector.
  
  % Coordinate ordering chosen to align with plots in cited papers.  This
  % results in the odd situation where the algebraically constrained
  % variable is first.
  E = y(1,:);
  Ep = y(2,:);
  E_fd = y(3,:);
  
  Ep_dot = (E_fd -(p.x + p.x_d) / p.xp * Ep ...
            + ((p.x_d - p.xp_d) / p.xp) * (E.^2 + p.xp * p.Q) ./ Ep) / p.Tp_d0;
  
  E_G = sqrt((p.x * p.P)^2 + (p.x * p.Q + E.^2).^2) ./ E;
  E_fd_dot = -(1/p.T) * ((E_fd - p.E0_fd) + p.K * (E_G - p.E_r));

  % The algebraic constraint.
  g = Ep.^2 .* E.^2 - (p.xp * p.P)^2 - (p.xp * p.Q + E.^2).^2;
  
  % Pack it all back up.  Don't forget the odd variable ordering.
  % Multiply by negative one since we are doing the backward reach set.
  ydot = -[ g; Ep_dot; E_fd_dot ];
  
  
  
%---------------------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%---------------------------------------------------------------------------
function ydot = dynamicsODE(t, y, p)
% dynamicsODE: implements an ODE version of single generator and load system.
%
% p is a structure containing the system parameters.
%
% This model assumes that Q = Q_0 + H E + B E^2 = Q_0 is constant
% (ie H = B = 0).  Modification would involve both replacing p.Q below
% with a state dependent vector AND rederiving E_dot.
  
  % Coordinate ordering chosen to align with plots in cited papers.  This
  % results in the odd situation where the algebraically constrained
  % variable is first.
  E = y(1,:);
  Ep = y(2,:);
  E_fd = y(3,:);
  
  
  Ep_dot = (E_fd -(p.x + p.x_d) / p.xp * Ep ...
            + ((p.x_d - p.xp_d) / p.xp) * (E.^2 + p.xp * p.Q) ./ Ep) / p.Tp_d0;
  
  E_G = sqrt((p.x * p.P)^2 + (p.x * p.Q + E.^2).^2) ./ E;
  E_fd_dot = -(1/p.T) * ((E_fd - p.E0_fd) + p.K * (E_G - p.E_r));
  
  % After taking the derivative of the algebraic constraint with respect
  % to time, we can extract an explicit equation for E_dot.
  E_dot = (E .* Ep .* Ep_dot) ./ (2 * E.^2 + 2 * p.xp * p.Q - Ep.^2);

  % Check whether the constraint remains satisfied.
  %g = Ep.^2 .* E.^2 - (p.xp * p.P)^2 - (p.xp * p.Q + E.^2).^2;
  %constraint_diff = g - p.g0
  
  % Pack it all back up.  Don't forget the odd variable ordering.
  % Multiply by negative one since we are doing the backward reach set.
  ydot = -[ E_dot; Ep_dot; E_fd_dot ];


  
%---------------------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%---------------------------------------------------------------------------
function [ value, isterminal, direction ] = detectSingular(t, y, p)
% detectSingular: Detects that the trajectory is near the singular surface.
%
% The singular surface is the surface where the constraint is singular with
% respect to E, so that we can no longer solve for E given Ep and E_fd by
% the implicit surface function.
%
% Near the singular surface, the DAE integrators seem to have a hard time
% (failure to maintain accuracy), so we'll just terminal the simulation
% if they approach the surface.
%
% We actually only detect the singular surface in the variable E, even
% though we also have a value for its position in Ep space.
  
  % Unpack the relevant state vector(s).
  E = y(1,:);

  % How close will we allow?
  too_close = 0.01;
  
  % We simulate only for value of E greater than the singular surface, so
  % we can use that to detect getting too close.
  value = E - p.E_sing - too_close;

  isterminal = 1;
  direction = -1;
  

  
%---------------------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%---------------------------------------------------------------------------
function [ E_sing, Ep_sing ] = visualizeSurfaces(p)
% visualizeSurfaces: algebraic constraint and singular surface visualization.
%
% This function creates a 3D surface visualization of the constraint surface
% for the DAE, and also plots the line where the algebraic constraint
% becomes singular with respect to E (and so you cannot solve for E by the
% implicit function theorem).  The DAE solvers, and certainly the ODE
% version of the problem, will not work well near this line.
%  
% p is the structure containing the problem parameters.
%
% This routine makes heavy use of the ToolboxLS kernel code.
  
% Create the grid.  Dimensions are in the order E, E', E_fd.
g.dim = 3;
g.N = 50;
g.min = [ 0.5; 0.8; -1 ];
g.max = [ 1.7; 1.8; +7 ];
g.bdry = @addGhostExtrapolate;
g = processGrid(g);

% Rename the grid node coordinates for easy reference.
E = g.xs{1};
Ep = g.xs{2};
E_fd = g.xs{3};

% The algebraic constraint g.
constraint = Ep.^2 .* E.^2 - (p.xp * p.P)^2 - (p.xp * p.Q + E.^2).^2;

% Surface visualization.
hC = visualizeLevelSet(g, constraint, 'surface', 0);
hold on;
set(hC, 'FaceColor', 'green', 'FaceAlpha', 0.3);

% Singular surface (for the case Q = Q_0).
E_sing = sqrt(p.xp * sqrt(p.P^2 + p.Q^2));
Ep_sing = sqrt(2 * p.xp * (p.Q + sqrt(p.P^2 + p.Q^2)));
E_fd_sing = [ g.min(3); g.max(3) ];

hS = plot3(E_sing * ones(2,1), Ep_sing * ones(2,1), E_fd_sing, 'k-');
set(hS, 'LineWidth', 2);
