function [ t, y ] = simulateToyDAE(show_plots)
% simulateToyDAE: simulates a toy DAE system.
%
%   [ t, y ] = simulateToyDAE(show_plots)
%
% This function simulates a simple 2D DAE system using Matlab's ode15s
% routine and a singular mass matrix.  It would be a script file, except
% that Matlab has mental problems running DAEs (and maybe ODEs?) with an
% inline function as the right hand side.  Therefore, there is a subfunction
% and a requirement for making the main routine a function.
%
% Input Parameters:
%
%   show_plots: Boolean.  Should the plot of the DAE solution be shown?
%   Default is true if there are no output arguments, and false
%   otherwise.
%
% Output Parameters (essentially the return values from ode15s):
%
%   t: Vector.  Sample times for the DAE solution in ys.
%
%   y: Array.  Solution to DAE.  Each row corresponds to the solution at
%   a time given in the corresponding row of ts.

% Copyright 2007 Ian M. Mitchell (mitchell@cs.ubc.ca).
% This software is used, copied and distributed under the licensing 
%   agreement contained in the file LICENSE in the top directory of 
%   the distribution.
%
% Ian Mitchell, 8/10/2007

  if(nargin < 1)
    if(nargout == 0)
      show_plots = 1;
    else
      show_plots = 0;
    end
  end
    
  % Time interval.  Equal spacing to see that the system slows down on the
  % left hand side of a phase space plot.
  tspan = [ 0, 3 ];

  % Mass matrix.
  M = [ 0, 0; 0, 1 ];

  % df/dy (Jacobian).
  J = inline('[ +1, -pi * cos(pi * y(2)); +1, 0 ]', 't', 'y');

  % Regular initial condition.
  y0 = [ 0; 0 ];

  % Initial slope (to help the DAE solver).
  yp0 = [ 2; 0 ];

  % Set up the integrator to handle DAEs.
  options = odeset('Mass', M, 'MassSingular', 'yes', ...
                   'Jacobian', J, 'Vectorized', 'on', 'InitialSlope', yp0);

  % Give it a whirl.
  [ t, y ] = ode15s(@dynamics, tspan, y0, options);

  % State vs time, to show slow periods on right hand side.
  if show_plots
    figure;
    plot(t, y(:,1), t, y(:,2));  
    legend('y_1', 'y_2'); xlabel('t'); ylabel('y');
    
    % Phase Space.
    figure;
    plot(y(:,1), y(:,2), 'b-');  xlabel('y_1');  ylabel('y_2');
    hold on;
    tt = (0:0.25:3)';
    yt = interp1(t, y, tt);
    plot(yt(:,1), yt(:,2), 'b*');
    axis image
  end

%---------------------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%---------------------------------------------------------------------------
function ydot = dynamics(t, y)
% For some stupid reason, the DAE integrators don't work when f(t,y) is an
% inline function.  Furthermore, they fail with a switch case problem
% regarding the mass matrix, which makes no sense.  And to top it off, in
% Matlab 6.5 the line number reported in the error in "odemass.m" is wrong
% (the reported line is an if statement; the actual error occurs further
% down the file in the final switch statement).
%
% Consequently, I've coded up this silly little routine as a subfunction.
% Maybe in Matlab 7x I could get away with an anonymous function.

  ydot = zeros(size(y));

  ydot(1) = y(1) - sin(pi * y(2));
  ydot(2) = 1.5 + y(1);
